# -*- coding: utf-8 -*-
"""Some test fixtures"""
import numpy as np
import pytest


@pytest.fixture()
def make_dataset():
    """Make a dataset with three targets"""
    return np.arange(0, 10, 1), np.arange(10, 20, 1)


@pytest.fixture()
def make_data_dictionary():
    """Create a data dictionary"""
    return {
        "x": {"data": [1, 2, 3], "type": "INDEPENDENT", "unit": "cm"},
        "y": {"data": [3, 2, 3], "type": "DEPENDENT", "unit": "h"},
        "z": {"data": [3, 2, 3], "type": "DEPENDENT", "unit": "kg"},
    }



# -*- coding: utf-8 -*-
"""Deal with n-tuple jcamps"""
import numpy as np

from .constants import END_STRING, NTUPLE_HEADER, NTUPLE_PAGE_HEADER, SYMBOL_LIST_NTUPLE
from .utils import _format_additional_header


def _process_data_dict(
    data_dictionary_key: str, data_dictionary_values: dict, i: int = 0
) -> dict:
    try:
        name = data_dictionary_values["name"].replace(r" *\[.*", "")
    except KeyError:
        name = ""
    try:
        unit = data_dictionary_values["unit"].replace(r".*\[(.*)\].*", "$1")
    except KeyError:
        unit = ""

    try:
        symbol = data_dictionary_values["symbol"]
    except KeyError:
        symbol = data_dictionary_key

    try:
        var_name = data_dictionary_values["name"]
    except KeyError:
        var_name = name if name != "" else data_dictionary_key

    try:
        var_type = data_dictionary_values["type"].upper()
        if var_type not in ["INDEPENDENT", "DEPENDENT"]:
            raise ValueError()
    except ValueError or KeyError:  # pylint:disable=binary-op-exception
        if i == 0:
            var_type = "INDEPENDENT"
        else:
            var_type = "DEPENDENT"

    maximum = np.max(data_dictionary_values["data"])
    minimum = np.min(data_dictionary_values["data"])

    return {
        data_dictionary_key: {
            "data": data_dictionary_values["data"],
            "type": var_type,
            "unit": unit,
            "maximum": maximum,
            "minimum": minimum,
            "symbol": symbol,
            "var_name": var_name,
            "dimension": len(data_dictionary_values["data"]),
            "name": name,
        }
    }


def from_dict(  # pylint:disable=too-many-arguments,too-many-locals
    data_dictionary: dict,
    title: str = "",
    owner: str = "",
    origin: str = "",
    data_type: str = "",
    meta: dict = None,
) -> str:
    """[summary]

    Args:
        data_dictionary (dict): a nested dictionary of form
            {
                'x' : {
                    'data': [],
                    'type': 'INDEPENDENT'
                    'unit': ''
                }
            }
        title (str, optional): Defaults to "".
        owner (str, optional):  Defaults to "".
        origin (str, optional): Defaults to "".
        data_type (str, optional):  Defaults to "".
        meta (dict, optional):  Defaults to None.

    Returns:
        str: [description]
    """
    var_names = []
    units = []
    symbols = []
    var_dims = []
    datas = []
    types = []

    header = NTUPLE_HEADER.format(
        title=title, dataType=data_type, owner=owner, origin=origin
    ) + _format_additional_header(meta)

    for counter, data_dict_tuple in enumerate(data_dictionary.items()):
        data_dictionary_key, data_dictionary_values = data_dict_tuple
        new_data_dict = _process_data_dict(
            data_dictionary_key, data_dictionary_values, counter
        )
        var_names.append(new_data_dict[data_dictionary_key]["var_name"])
        units.append(new_data_dict[data_dictionary_key]["unit"])
        symbols.append(new_data_dict[data_dictionary_key]["symbol"])
        datas.append(new_data_dict[data_dictionary_key]["data"])
        var_dims.append(str(new_data_dict[data_dictionary_key]["dimension"]))
        types.append(new_data_dict[data_dictionary_key]["type"])

    ntuple_page_header = NTUPLE_PAGE_HEADER.format(
        varName=",".join(var_names),
        symbol=",".join(symbols),
        varType=",".join(types),
        varDim=",".join(var_dims),
        units=",".join(units),
        dataType=data_type,
    )

    peak_table_header = SYMBOL_LIST_NTUPLE.format(symbols=",".join(symbols))

    point_lists = "\n".join(["\t".join([str(p) for p in points]) for points in datas])

    jcamp_string = (
        header
        + ntuple_page_header
        + peak_table_header
        + point_lists
        + "\n"
        + END_STRING
    )

    return jcamp_string



# -*- coding: utf-8 -*-
"""Convert simple x/y data to jcamp"""
from typing import Iterable, Tuple, Union

import numpy as np

from .constants import HEADER
from .utils import _format_additional_header


def from_xy(  # pylint:disable=too-many-arguments, too-many-locals
    data: Tuple[Iterable[Union[float, int]], Iterable[Union[float, int]]],
    title: str = "",
    owner: str = "",
    origin: str = "",
    data_type: str = "",
    x_units: str = "",
    y_units: str = "",
    meta: dict = None,
) -> str:
    """Convert a tuple of x/y data to a JCAMP string

    Args:
        data (Tuple[Iterable[Union[float, int]],
            Iterable[Union[float, int]]]): Tuple of data
        title (str, optional): Defaults to "".
        owner (str, optional): Defaults to "".
        origin (str, optional): Defaults to "".
        data_type (str, optional): Defaults to "".
        x_units (str, optional):  Defaults to "".
        y_units (str, optional):  Defaults to "".
        meta (dict, optional): Additional metadata for the header. Defaults to None.

    Raises:
        ValueError: [description]

    Returns:
        str: [description]
    """
    try:
        x_data, y_data = data
        first_x = np.min(x_data)
        first_y = np.min(y_data)
        last_x = np.max(x_data)
        last_y = np.max(y_data)
    except ValueError:
        raise ValueError(
            "You need to provide a tuple of iterables of floats or integers"
        )

    points = [f"{x_point}  {y_point}" for x_point, y_point in zip(x_data, y_data)]

    header_dict = {
        "dataType": data_type,
        "origin": origin,
        "owner": owner,
        "title": title,
        "xUnits": x_units,
        "yUnits": y_units,
        "firstX": first_x,
        "firstY": first_y,
        "lastX": last_x,
        "lastY": last_y,
        "points": len(points),
    }

    header = HEADER.format(**header_dict)
    additional_header = _format_additional_header(meta)
    point_list = "\n".join(points)

    full_jcamp = (
        header + additional_header + "##PEAK TABLE=(XY..XY)\n" + point_list + "\n##END"
    )

    return full_jcamp



# -*- coding: utf-8 -*-
"""Helper functions"""
from typing import Union

from .constants import ADDITIONAL_META


def _format_additional_header(meta: Union[dict, None]) -> str:
    """Create a multiline string for all the metadata

    Args:
        meta (Dict): Dictionary with metadata

    Raises:
        ValueError: If meta is not a dictionary

    Returns:
        str: multiline string for the JCAMP string
    """
    additional_header = ""
    if meta is not None:
        try:
            for key, value in meta.items():
                additional_header += ADDITIONAL_META.format(key=key, value=value)
        except AttributeError:
            raise ValueError(
                "If you provide metadata, you need to provide it as dictionary"
            )

    return additional_header



# -*- coding: utf-8 -*-
"""Some constant parts of JCAMP files"""

HEADER = """##TITLE={title}
##JCAMP-DX=4.24
##DATA TYPE={dataType}
##ORIGIN={origin}
##OWNER={owner}
##XUNITS={xUnits}
##YUNITS={yUnits}
##FIRSTX={firstX}
##LASTX={lastX}
##FIRSTY={firstY}
##LASTY={lastY}
##NPOINTS={points}\n"""

NTUPLE_HEADER = """##TITLE={title}
##JCAMP-DX=6.00
##DATA TYPE={dataType}
##ORIGIN={origin}
##OWNER={owner}\n"""

NTUPLE_PAGE_HEADER = """##NTUPLES= {dataType}
##VAR_NAME=  {varName}
##SYMBOL=    {symbol}
##VAR_TYPE=  {varType}
##VAR_DIM=   {varDim}
##UNITS=     {units}
##PAGE= N=1\n"""

SYMBOL_LIST_NTUPLE = "##DATA TABLE= ({symbols}..{symbols}), PEAKS\n"


ADDITIONAL_META = "##${key}={value}\n"

END_STRING = "##END"



# -*- coding: utf-8 -*-
"""Testing the from_xy module"""
import pytest

from pytojcamp import from_xy


def test_from_xy(make_dataset):
    """Test conversion from x/y data"""
    x, y = make_dataset  # pylint:disable=invalid-name

    with pytest.raises(ValueError):
        from_xy(x)

    jcamp_string = from_xy(
        (x, y),
        "TestTile",
        "TestOwner",
        "TestOrigin",
        "TestData",
        "cm",
        "kg",
        {"hello": "world"},
    )

    expected_string = """##TITLE=TestTile
##JCAMP-DX=4.24
##DATA TYPE=TestData
##ORIGIN=TestOrigin
##OWNER=TestOwner
##XUNITS=cm
##YUNITS=kg
##FIRSTX=0
##LASTX=9
##FIRSTY=10
##LASTY=19
##NPOINTS=10
##$hello=world
##PEAK TABLE=(XY..XY)
0  10
1  11
2  12
3  13
4  14
5  15
6  16
7  17
8  18
9  19
##END"""

    assert jcamp_string == expected_string

    with pytest.raises(ValueError):
        jcamp_string = from_xy(
            (x, y),
            "TestTile",
            "TestOwner",
            "TestOrigin",
            "TestData",
            "cm",
            "kg",
            "meta",
        )



# -*- coding: utf-8 -*-
"""Test the ntuple conversion"""
from pytojcamp.from_dict import from_dict


def test_from_dict(make_data_dictionary):
    """Test the conversion based on a data dict"""
    jcamp_string = from_dict(make_data_dictionary)
    expected_string = """##TITLE=
##JCAMP-DX=6.00
##DATA TYPE=
##ORIGIN=
##OWNER=
##NTUPLES= 
##VAR_NAME=  x,y,z
##SYMBOL=    x,y,z
##VAR_TYPE=  INDEPENDENT,DEPENDENT,DEPENDENT
##VAR_DIM=   3,3,3
##UNITS=     cm,h,kg
##PAGE= N=1
##DATA TABLE= (x,y,z..x,y,z), PEAKS
1\t2\t3
3\t2\t3
3\t2\t3
##END"""
    assert jcamp_string == expected_string



# -*- coding: utf-8 -*-
"""pytojcamp package to write JCAMP files from Python"""
from ._version import get_versions
from .from_dict import from_dict
from .from_xy import from_xy

__version__ = get_versions()["version"]
del get_versions



# -*- coding: utf-8 -*-
"""Convert Python objects to jcamp"""

from setuptools import find_packages, setup

import versioneer

with open("requirements.txt", "r") as fh:
    REQUIREMENTS = [line.strip() for line in fh]

with open("README.md") as f:
    README = f.read()


setup(
    name="pytojcamp",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=find_packages(),
    url="",
    license="MIT",
    long_description=README,
    long_description_content_type="text/markdown",
    python_requires=">=3.6",
    install_requires=REQUIREMENTS,
    extras_require={
        "testing": ["pytest", "pytest-cov<2.11"],
        "docs": ["guzzle_sphinx_theme"],
        "pre-commit": [
            "pre-commit",
            "black",
            "prospector",
            "pylint",
            "versioneer",
            "isort",
            "gitchangelog",
        ],
    },
    author="Kevin M. Jablonka",
    author_email="kevin.jablonka@epfl.ch",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Physics",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
